## -*- asm -*- ##############################################################
#                                                                           #
#                 GNAT RUN-TIME LIBRARY (GNARL) COMPONENTS                  #
#                                                                           #
#                               H A N D L E R                               #
#                                                                           #
#                               Assembly File                               #
#                                                                           #
#                         Copyright (C) 2012 AdaCore                        #
#                                                                           #
#  GNAT is free software;  you can  redistribute it  and/or modify it under #
#  terms of the  GNU General Public License as published  by the Free Soft- #
#  ware  Foundation;  either version 3,  or (at your option) any later ver- #
#  sion.  GNAT is distributed in the hope that it will be useful, but WITH- #
#  OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY #
#  or FITNESS FOR A PARTICULAR PURPOSE.                                     #
#                                                                           #
#  As a special exception under Section 7 of GPL version 3, you are granted #
#  additional permissions described in the GCC Runtime Library Exception,   #
#  version 3.1, as published by the Free Software Foundation.               #
#                                                                           #
#  You should have received a copy of the GNU General Public License and    #
#  a copy of the GCC Runtime Library Exception along with this program;     #
#  see the files COPYING3 and COPYING.RUNTIME respectively.  If not, see    #
#  <http://www.gnu.org/licenses/>.                                          #
#                                                                           #
#############################################################################

	.syntax unified
	.cpu cortex-m3
	.thumb

	.section .vectors,"a"
	.globl	__vectors
	.p2align 8
__vectors:
	.word   0           /* stack top address */
	.word   handler     /* 1 Reset.  */
	.word   handler     /* 2 NMI.  */
	.word   fault	    /* 3 Hard fault. */
	.word   handler     /* 4 Mem manage.  */
	.word   handler     /* 5 Bus fault.  */
	.word   handler     /* 6 Usage fault.  */
	.word   handler     /* 7 reserved.  */
	.word   handler     /* 8 reserved.  */
	.word   handler     /* 9 reserved.  */
	.word   handler     /* 10 reserved.  */
	.word   handler     /* 11 SVCall.  */
	.word   handler     /* 12 Reserved for Debug.  */
	.word   handler     /* 13 Reserved.  */
	.word   pend_sv     /* 14 PendSV.  */
	.word   handler     /* 15 Systick.  */
	.word   handler     /* 16 IRQ0: GPIO A.  */
	.word   handler     /* 17 IRQ1: GPIO B.  */
	.word   handler     /* 18 IRQ2.  */
	.word   handler     /* 19 IRQ3.  */
	.word   handler     /* 20 IRQ4.  */
	.word   handler     /* 21 IRQ5.  */
	.word   handler     /* 22 IRQ6.  */
	.word   handler     /* 23 IRQ7.  */
	.word   handler     /* 24 IRQ8.  */
	.word   handler     /* 25 IRQ9.  */
	.word   handler     /* 26 IRQ10.  */
	.word   handler     /* 27 IRQ11.  */
	.word   handler     /* 28 IRQ12.  */
	.word   handler     /* 29 IRQ13.  */
	.word   handler     /* 30 IRQ14.  */
	.word   handler     /* 31 IRQ15.  */
	.word   handler     /* 32 IRQ16.  */
	.word   handler     /* 33 IRQ17.  */
	.word   handler     /* 34 IRQ18.  */
	.word   handler     /* 35 IRQ19.  */
	.word   handler     /* 36 IRQ20.  */
	.word   handler     /* 37 IRQ21.  */
	.word   handler     /* 38 IRQ22.  */
	.word   handler     /* 39 IRQ23.  */
	.word   handler     /* 40 IRQ24.  */
	.word   handler     /* 41 IRQ25.  */
	.word   handler     /* 42 IRQ26.  */
	.word   handler     /* 43 IRQ27.  */
	.word   handler     /* 44 IRQ28.  */
	.word   handler     /* 45 IRQ29.  */
	.word   handler     /* 46 IRQ30.  */
	.word   handler     /* 47 IRQ31.  */
	.word   handler     /* 48 IRQ32.  */
	.word   handler     /* 49 IRQ33.  */
	.word   handler     /* 50 IRQ34.  */
	.word   handler     /* 51 IRQ35.  */
	.word   handler     /* 52 IRQ36.  */
	.word   handler     /* 53 IRQ37.  */
	.word   handler     /* 54 IRQ38.  */
	.word   handler     /* 55 IRQ39.  */
	.word   handler     /* 56 IRQ40.  */
	.word   handler     /* 57 IRQ41.  */
	.word   handler     /* 58 IRQ42.  */
	.word   handler     /* 59 IRQ43.  */

	.text

	.thumb_func
handler:
	@ These registers are saved in the current stack:
	@ xPSR
	@ PC
	@ LR
	@ R12
	@ R3
	@ R2
	@ R1
	@ R0

	@ Load handler address
	mrs	r0, PSR
	movw	r1,#:lower16:__gnat_cortexm3_exception_handlers
	movt	r1,#:upper16:__gnat_cortexm3_exception_handlers
	and	r0,r0,0x3f
	ldr	r2,[r1, r0, LSL #2]

	@ Execute the handler
	push	{r4,lr}
	blx	r2

	@ Check if context switch is needed
	bl      context_switch_needed
	pop	{r4,lr}
	cmp	r0,0
	it	eq
	bxeq	lr

	@ Things are becoming interesting
	@ We need to switch context and to exit from the interrupt
	@
	@ Method 1:
	@ The registers saved (during the interrupt) are fetched and written
	@  into the context area
	@ The stack is patched to jump to context switch
	@ Return from interrupt (this will switch the context, and eventually
	@  switching back to this thread)
	@ Execute PendSV (Prio 0)
	@ Patch the stack to return back to the original state (except IPSR)
	@ PB: what is the right value of IPSR ?
	@
	@ Method 2:
	@ Create an exception frame on the MSP
	@ EXC_RETURN to main stack
	@ context switch (and back to this one)
	@ execute PendSV (prio 0)
	@ destroy the exception frame
	@ EXC_RETURN

	@ We use method 2
	push	{r3,lr}	@ Save lr. ??? To optimize
	@ Create an exception frame, clear aligner
	tst	lr,4	@ Test bit 2, set for return on the process stack
	ite	ne
	mrsne	r1, PSP
	mrseq	r1, MSP
	ldr	r3,[r1,#28]
	bic	r3,r3,#0x200
	adr.w	r2,Lctsw+1	@ Thumb mode
	push	{r0,r1,r2,r3} @ For r12,lr,pc,xpsr
	push	{r0,r1,r2,r3}

	@ Return to thread mode, using MSP
	mvn	lr,#0x6
	bx	lr
Lctsw:	@ Switch to PSP
	mrs	r2,CONTROL
	orr	r2,r2,#2
	msr	CONTROL,r2
	@ Context switch
	blx	context_switch
	@ Switch to MSP
	mrs	r2,CONTROL
	bic	r2,r2,#2
	msr	CONTROL,r2
	@ Restore lr
	pop	{r3,lr}
	@ Trigger PendSV
	movw	r1,#0xed04
	movt	r1,#0xe000
	movw	r2,#0x0000
	movt	r2,#0x1000
0:	str	r2,[r1]
	cpsie	i		@ Enable interrupts.  What about normal case ???
	b	0b

	.size handler, . - handler

	.thumb_func
pend_sv:
	@ The context is saved on the MSP
	ldr	lr,[sp,#20]
	ldr	r0,[sp,#28]
	add	sp,sp,#32	@ the stack is aligned on 8 bytes

	@ Need to patch saved PSR
	tst	lr,4	@ Test bit 2, set for return on the process stack
	ite	ne
	mrsne	r1, PSP
	mrseq	r1, MSP
	ldr	r2,[r1,#28]
	and	r0,r0,#0x3f
	bic	r2,r2,#0x3f
	orrs	r0,r2
	str	r0,[r1,#28]
	bx	lr
	.size	pend_sv, . - pend_sv

	.thumb_func
fault:	b	fault
